/*
 * CDemoIO.c - Demonstrates using user-specified I/O functions that can be used
 * ~~~~~~~~~   for reading image in memory or that is created on the fly.
 *             This simple example reads a file stored on a disk.
 *             It simply loads and displays an image. The image gets automatically
 *             resized to the current window size using PVSetStretchParameters().
 *
 * Author: Jan Patera, 2009
 */

#include <windows.h>
#include "PVW32DLL.h"

char        szFrameClass [] = "PVW32Demo";
int         loaded=FALSE;
LPPVHandle  PVHandle;
PVImageInfo pvii;

/* MySeekFunc - user-specified seek function */
DWORD  WINAPI MySeekFunc(void *AppSpecific, LONG NewPos, int Origin)
{
  /* Origin can be any of FILE_BEGIN/FILE_CURRENT/FILE_END */
  return _llseek((HFILE)AppSpecific, NewPos, Origin);
}

/* MyReadFunc - user-specified input read function */
DWORD  WINAPI MyReadFunc(void *AppSpecific, void *pData, DWORD Size)
{
  return _lread((HFILE)AppSpecific, pData, Size);
}

/* This callback function displays the decompression progress
 */
BOOL WINAPI ProgressProcedure(int progress, void *hwnd)
{
  char s[64];

  wsprintf(s, "PictView32 DLL Shrink Demo (%i%%)", progress);
  SetWindowText((HWND)hwnd, s);
  return FALSE;
}

/* The main window procedure
 */
LRESULT CALLBACK FrameWndProc(HWND hwnd, UINT message, WPARAM wParam,
     LPARAM lParam)
{
  PAINTSTRUCT  ps;
  HDC          hdc;
  RECT         rect;

  switch (message)
  {
    case WM_PAINT:
            hdc = BeginPaint(hwnd, &ps);
            if (!loaded)
             {
               PVCODE   code;
               RECT     rect;

               loaded = TRUE;
               rect.top = rect.left = 0; // paint to top-left corner
               rect.right = ps.rcPaint.right; // do not paint more than necessary
               rect.bottom = ps.rcPaint.bottom;
               code = PVReadImage2(PVHandle, hdc, &rect, ProgressProcedure, hwnd, 0);
               if (code != PVC_OK)
                {
                  MessageBox(hwnd, PVGetErrorText(code), "Error calling PVW32Cnv.DLL", MB_ICONEXCLAMATION);
                  if (code != PVC_CANCELED)
                    PostMessage(hwnd, WM_CLOSE, 0, 0);
                }
             }
            else
             {
               SetViewportOrgEx(hdc, -GetScrollPos(hwnd, SB_HORZ),
                -GetScrollPos(hwnd, SB_VERT), NULL);
               PVDrawImage(PVHandle, hdc, 0, 0, NULL);
             }
            EndPaint(hwnd, &ps);
            return 0;
    case WM_SIZE:
            GetClientRect(hwnd, &rect);
            if (PVHandle)
            {
              PVSetStretchParameters(PVHandle, 
               rect.right == pvii.Width ? PV_STRETCH_NO : rect.right, 
               rect.bottom == pvii.Height ? PV_STRETCH_NO : rect.bottom, COLORONCOLOR);
              InvalidateRect(hwnd, NULL, FALSE);
            }

    case WM_QUERYENDSESSION:
    case WM_CLOSE:
            break;

    case WM_DESTROY:
            PostQuitMessage (0);
            return 0;
  }

  return DefWindowProc(hwnd, message, wParam, lParam);
}

int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow)
{
  MSG      msg;
  WNDCLASS wndclass;
  PVCODE   code;
  HWND     hWnd;
  PVOpenImageExInfo oii;
  LONG     hFile;

  wndclass.style         = 0;
  wndclass.lpfnWndProc   = FrameWndProc;
  wndclass.cbClsExtra    = 0;
  wndclass.cbWndExtra    = 0;
  wndclass.hInstance     = hInstance;
  wndclass.hIcon         = 0;
  wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
  wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
  wndclass.lpszMenuName  = NULL;
  wndclass.lpszClassName = szFrameClass;

  RegisterClass (&wndclass);

  if (!lpszCmdLine || !*lpszCmdLine)
    return MessageBox(0, "Usage: CDemoIO.exe image_file", "PictView32 DLL user I/O Demo", 0);

  pvii.Width = pvii.Height = CW_USEDEFAULT;

  /* The input image is read in using our own input functions */
  oii.cbSize = sizeof(PVOpenImageExInfo);
  oii.Flags = PVOF_USERDEFINED_INPUT;
  oii.ReadFunc = MyReadFunc;
  oii.SeekFunc = MySeekFunc;
  hFile = _lopen(lpszCmdLine, OF_READ);

  if (hFile == -1)
  {
    MessageBox(0, "Unable to open input file", "Error converting file", MB_ICONEXCLAMATION);
    return 0;
  }

  oii.Handle = (HANDLE)hFile; /* Argument to MyReadFunc and MySeekFunc */
  /* Get the file size */
  oii.DataSize = _llseek(hFile, 0, FILE_END);
  _llseek(hFile, 0, FILE_BEGIN);

  code = PVOpenImageEx(&PVHandle, &oii, &pvii, sizeof(pvii));
  if (PVC_OK != code)
  {
    MessageBox(0, PVGetErrorText(code), "Error calling PVW32Cnv.DLL", MB_ICONEXCLAMATION);
    return 0;
  }

  PVSetBkHandle(PVHandle, (HBRUSH)(COLOR_WINDOW+1));
  hWnd = CreateWindow(szFrameClass, "PictView32 DLL I/O Demo",
    WS_OVERLAPPEDWINDOW | WS_VSCROLL | WS_HSCROLL,
    CW_USEDEFAULT, CW_USEDEFAULT,
    pvii.Width+2*GetSystemMetrics(SM_CXFRAME),
    pvii.Height+2*GetSystemMetrics(SM_CYFRAME)+GetSystemMetrics(SM_CYCAPTION),
    NULL, NULL, hInstance, NULL);

  ShowWindow(hWnd, SW_SHOWNORMAL);
  ShowScrollBar(hWnd, SB_BOTH, FALSE);
  UpdateWindow(hWnd);

  while (GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  PVCloseImage(PVHandle);
  _lclose(hFile);
  return msg.wParam;
}
