/*
 * MakeThumb.cpp - Demonstrates basic usage of PVW32Cnv.DLL.
 * ~~~~~~~~~~~~~   Creates a thumbnail image of at most 160x120 pixels
 *                 from the image specified on the command-line.
 *                 The thumbnail format is 32bit BMP.
 *                 The main purpose of this example is to show
 *                 how to create thumbnails in memory using
 *                 user-specified seek & write functions.
 *
 * NOTE:         Must be linked with PVW32Cnv.lib as a console application.
 *
 * Author: Jan Patera, 2005-2009
 */

#include <windows.h>
#include <stdio.h>
#include <pvw32dll.h>

#define thumbWidth   160
#define thumbHeight  120

typedef struct _OutputFileInfo {
  FILE              *f;
} sOutputFileInfo, *psOutputFileInfo;

/* MySeekFunc - used for export; actually not used when saving to PVF_RAW */
DWORD  WINAPI MySeekFunc(void *AppSpecific, LONG NewPos, int Origin)
{
  psOutputFileInfo pOFI = (psOutputFileInfo)AppSpecific;
  int              base;

  switch (Origin) {
     case FILE_BEGIN:
       base = SEEK_SET;
       break;
     case FILE_CURRENT:
       base = SEEK_CUR;
       break;
     case FILE_END:
       base = SEEK_END;
       break;
  }
  fseek(pOFI->f, NewPos, base);
  return ftell(pOFI->f);
}

/* MyWriteFunc - used for export
   When saving to PVW_RAW, untegral number of image lines is sent at once - no line is broken
   into several parts.
   Expected return value: # of written bytes.
   Instead of dummy writing to a file you can put here your own shrinking of the image.
 */
DWORD  WINAPI MyWriteFunc(void *AppSpecific, void *pData, DWORD Size)
{
  psOutputFileInfo pOFI = (psOutputFileInfo)AppSpecific;

  return fwrite(pData, 1, Size, pOFI->f);
}


int main(int argc, char *argv[])
{
  PVSaveImageInfo   sii;
  LPPVHandle        hPVImage;
  PVImageInfo       pvii;
  PVOpenImageExInfo pvoi;
  PVCODE            code;
  sOutputFileInfo   ofi;
  BITMAPFILEHEADER  bfh = {'MB', 0, 0, 0, sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)};
  BITMAPINFOHEADER  bih = {sizeof(BITMAPINFOHEADER), 0, 0, 1, 32, 0, 0, 0, 0, 0, 0};

  if (argc != 3) {
     printf("Usage:\n  MakeThumb.exe  Image.ext   Thumbnail.bmp\n");
     return 1;
  }
  // Create the output file
  ofi.f = fopen(argv[2], "wb");
  if (!ofi.f) {
     printf("Error %d: cannot create output file %s\n", errno, argv[2]);
     return 2;
  }

  memset(&pvoi, 0, sizeof(pvoi));
  pvoi.cbSize = sizeof(pvoi);
  // PVFF_FAST: Discard/do not alloc all unnecessary info
  pvoi.Flags  = PVFF_FAST | PVOF_THUMBNAIL;
  pvoi.FileName = argv[1];

  // Open the file
  code = PVOpenImageEx(&hPVImage, &pvoi, &pvii, sizeof(pvii));
  if (code != PVC_OK) {
     fclose(ofi.f);
     printf("Error %d opening file %s:\n%s\n", code, argv[1], PVGetErrorText(code));
     // Unknown file format?
     return FALSE;
  }

  if (pvii.Flags & PVFF_ROTATE90) {
     // 90/270-deg rotated thumbnail, most likely EXIF JPEG
     DWORD tmp = pvii.Height;
     pvii.Height = pvii.Width;
     pvii.Width = tmp;
  }

  memset(&sii, 0, sizeof(sii));
  sii.cbSize = sizeof(sii);
  // PVF_BMP or any other format could be used as well
  sii.Format = PVF_RAW;
  sii.Compression = PVCS_NO_COMPRESSION;
  
  if (pvii.Flags & PVFF_BOTTOMTOTOP) {
     // Advanced technique to simplify the process of thumbnailizing.
     // When the image is read into memory in bottom-to-top order, then make
     // the output in the same order to avoid caching of entire image in memory
     // -> Image can be saved on the with small memory requirements.
     sii.Flags = PVSF_FLIP_VERT;
  }
  // Use PV_COLOR_TC32 to get 32bit data
  sii.Colors = PV_COLOR_TC32;
  sii.ColorModel = PVCM_RGB;

  // Image is saved using user-defined output functions
  sii.Flags |= PVSF_USERDEFINED_OUTPUT | PVFF_FAST;
  sii.WriteFunc = MyWriteFunc;
  sii.SeekFunc = MySeekFunc;

  if (pvii.Flags & PVFF_ROTATE90) {
     sii.Flags |= PVSF_ROTATE90;
  }
  if (pvii.Flags & PVFF_FLIP_HOR) {
     sii.Flags |= PVSF_FLIP_HOR;
  }
  // Make the thumbnail not larger than thumbWidth x thumbHeight pixels
  if ((pvii.Width > thumbWidth) || (pvii.Height > thumbHeight)) {
     sii.Width = thumbWidth;
     sii.Height = pvii.Height * sii.Width/pvii.Width;
     if (sii.Height > thumbHeight) {
        // Still too big
        sii.Height = thumbHeight;
        sii.Width = pvii.Width * sii.Height/pvii.Height;
     }
     // Ensure non-zero size
     if (!sii.Width) sii.Width = 1;
     if (!sii.Height) sii.Height = 1;
     bih.biWidth = sii.Width;
     bih.biHeight = sii.Height;
  } else {
     bih.biWidth = pvii.Width;
     bih.biHeight = pvii.Height;
  }
  bfh.bfSize = sizeof(bfh) + sizeof(bih) + bih.biWidth*bih.biHeight*4;
  if (!(pvii.Flags & PVFF_BOTTOMTOTOP)) {
     // Normal BMPs are stored bottom-to-top
     // Negative height indicates top-to-bottom order
     bih.biHeight = -bih.biHeight;
  }
  fwrite(&bfh, 1, sizeof(bfh), ofi.f); // BitmapFileHeader
  fwrite(&bih, 1, sizeof(bih), ofi.f); // BitmapInfoHeader
  code = PVSaveImage(hPVImage, NULL, &sii, NULL, (void*)&ofi, pvii.CurrentImage);

  if (code != PVC_OK) {
     printf("Error %d saving file %s:\n%s\n", code, argv[2], PVGetErrorText(code));
  }

  // Close the input file
  PVCloseImage(hPVImage);
  fclose(ofi.f);
  return 0;
}
