(*
 * MakeThumb.dpr - Demonstrates basic usage of PVW32Cnv.DLL.
 * ~~~~~~~~~~~~~   Creates a thumbnail image of at most 160x120 pixels
 *                 from the image specified on the command-line.
 *                 The thumbnail format is 32bit BMP.
 *                 The main purpose of this example is to show
 *                 how to create thumbnails in memory using
 *                 user-specified seek & write functions.
 *
 * NOTE:         Must be linked with PVW32Cnv.pas as a console application.
 *
 * Author: Jan Patera, 2006-2009
 *)

{$APPTYPE CONSOLE}
program MakeThumb;
uses Windows, SysUtils, pvw32cnv;

const thumbWidth   =160;
const thumbHeight  =120;

type
 POutputFileInfo = ^TOutputFileInfo;
 TOutputFileInfo = record
    f: Cardinal;
  end;

{ MySeekFunc - used for export; actually not used when saving to PVF_RAW }
function MySeekFunc(AppSpecific: DWORD; NewPos: DWORD; Origin: DWORD): DWORD; stdcall;
 var
  pOFI: POutputFileInfo;
begin
 pOFI := POutputFileInfo(AppSpecific);
 MySeekFunc := _llseek(pOFI^.f, NewPos, Origin);
end;

{ MyWriteFunc - used for export }
function MyWriteFunc(AppSpecific: DWORD; var pData; Size: DWORD): DWORD; stdcall;
 var
  pOFI: POutputFileInfo;
begin
 pOFI := POutputFileInfo(AppSpecific);

 MyWriteFunc := _lwrite(pOFI^.f, @pData, Size);
end;

var
  sii:  TPVSaveImageInfo;
  hPVImage: PPVHandle;
  pvii: TPVImageInfo;
  pvoi: TPVOpenImageExInfo;
  code: TPVCODE;
  ofi:  TOutputFileInfo;
  InpFileName, OutFileName: array[0..MAX_PATH] of char;
  Tmp:  DWORD;

const
  bfh: TBITMAPFILEHEADER = (bfType : $4d42; bfSize: 0; bfReserved1: 0; bfReserved2: 0; bfOffBits: $36);
  bih: TBITMAPINFOHEADER = (biSize: $28; biWidth: 0; biHeight: 0; biPlanes: 1; biBitCount: 32;
    biCompression: 0; biSizeImage: 0; biXPelsPerMeter: 0; biYPelsPerMeter: 0; biClrUsed: 0; biClrImportant: 0);

{Main program}
begin
  if ParamCount <> 2 then
   begin
    WriteLn('Usage:');
    WriteLn(' MakeThumb.exe  Image.ext   Thumbnail.bmp');
    Exit;
  end;
  { Create the output file }
  StrPCopy(OutFileName, ParamStr(2));
  ofi.f := _lcreat(OutFileName, 0);
  if ofi.f = HFILE_ERROR then
   begin
    WriteLn('Error ', GetLastError, ': cannot create output file ', ParamStr(1));
    Exit;
   end;

  pvoi.Size := sizeof(pvoi);
  { PVFF_FAST: Discard/do not alloc all unnecessary info }
  pvoi.Flags  := PVFF_FAST or PVOF_THUMBNAIL;
  StrPCopy(InpFileName, ParamStr(1));
  pvoi.FileName := InpFileName;

  { Open the file }
  code := PVOpenImageEx(hPVImage, @pvoi, @pvii, sizeof(pvii));
  if code <> PVC_OK then
   begin
    _lclose(ofi.f);
    WriteLn('Error ', code, ' opening file ', ParamStr(1), ':');
    WriteLn(PVGetErrorText(code));
    { Unknown file format? }
    Exit;
   end;

  if (pvii.Flags and PVFF_ROTATE90) <> 0 then
   begin
    { 90deg rotated thumbnail, most likely EXIF JPEG }
    tmp := pvii.Height;
    pvii.Height := pvii.Width;
    pvii.Width := tmp;
   end;

  FillChar(sii, sizeof(sii), 0);
  sii.Size := sizeof(sii);
  sii.Format := PVF_RAW;
  sii.Compression := PVCS_NO_COMPRESSION;

  if (pvii.Flags and PVFF_BOTTOMTOTOP) <> 0 then
   begin
    { When the image is read into memory in bottom-to-top order, then make
      the output in the same order to avoid caching of entire image in memory }
     sii.Flags := PVSF_FLIP_VERT;
   end;
  { Use PV_COLOR_TC32 to get 32bit data }
  sii.Colors := PV_COLOR_TC32;
  sii.ColorModel := PVCM_RGB;

  { Image is saved using user-defined output handling functions }
  sii.Flags := sii.Flags or PVSF_USERDEFINED_OUTPUT;
  sii.WriteFunc := MyWriteFunc;
  sii.SeekFunc := MySeekFunc;

  if (pvii.Flags and PVFF_ROTATE90) <> 0 then
   begin
    sii.Flags := sii.Flags or PVSF_ROTATE90;
   end;
  if (pvii.Flags and PVFF_FLIP_HOR) <> 0 then
   begin
    sii.Flags := sii.Flags or PVSF_FLIP_HOR;
   end;
  { Make the thumbnail not larger than thumbWidth x thumbHeight pixels }
  if (pvii.Width > thumbWidth) or (pvii.Height > thumbHeight) then
   begin
    sii.Width := thumbWidth;
    sii.Height := (pvii.Height * sii.Width) div pvii.Width;
    if sii.Height > thumbHeight then
     begin
      { Still too big }
       sii.Height := thumbHeight;
       sii.Width := (pvii.Width * sii.Height) div pvii.Height;
     end;
    { Ensure non-zero size }
    if sii.Width = 0 then sii.Width := 1;
    if sii.Height = 0 then sii.Height := 1;
    bih.biWidth := sii.Width;
    bih.biHeight := sii.Height;
   end
  else
   begin
    bih.biWidth := pvii.Width;
    bih.biHeight := pvii.Height;
   end;
  bfh.bfSize := SizeOf(bfh) + SizeOf(bih) + bih.biWidth*bih.biHeight*4;
  if (pvii.Flags and PVFF_BOTTOMTOTOP) = 0 then
   begin
    { Normal BMPs are stored bottom-to-top
      Negative height indicates top-to-bottom order }
    bih.biHeight := -bih.biHeight;
   end;
  _lwrite(ofi.f, @bfh, SizeOf(bfh)); { BitmapFileHeader }
  _lwrite(ofi.f, @bih, SizeOf(bih));  { BitmapInfoHeader }
  code := PVSaveImage(hPVImage, NIL, sii, NIL, @ofi, pvii.CurrentImage);

  if code <> PVC_OK then
   begin
    WriteLn('Error ', code, ' saving file ', ParamStr(2), ':');
    WriteLn(PVGetErrorText(code));
   end;

  { Close the input image }
  PVCloseImage(hPVImage);
  _lclose(ofi.f);
end.
